/***************************************************************\
      This is All For Chinese Input/Output Method Proposal

  Author: Brian Lin
  Copyright: Open Source, GPL, Copylefted

\***************************************************************/

#ifndef __IOME_H
#define __IOME_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#ifdef __cplusplus
extern "C" {
#endif

/*******************************************************\
  A4C IOME Configuration Proposal

  I strongly suggest IOME configuration should be unified.

  I recommend configuration file should be placed either
  in the /etc/iome/ directory 
  or 
  $HOME/.iome/ directory, 
  but not necessarily restrict.

  All For Chinese support multilingual configuration.
  
  system configuration file should be

  $HOME/.iome/configure   - System Configuration
   Keywords
     language definition - lang big5, gb2312
     configuration file  - configure iome.big5, iome.gb2312
     modules             - modules iome.modules
     display             - optional X DISPLAY NAME

  $HOME/.iome/conf.big5   - BIG5 Version Configuration
  $HOME/.iome/conf.gb2312 - GB Version Configuration

\*******************************************************/

typedef struct {
  char * key;
  char * value;
} iomeConfItem;

typedef struct {
  int            size;
  iomeConfItem * items;
} iomeConfig;

#define IOME_MAX_BUFF 2048

int iomeLoadConfiguration(char *FileName,iomeConfig *CONF);
int iomeFreeConfiguration(iomeConfig *CONF);

/* Utilities for Configuration */
int iomeGetLine(FILE *fp,char *buff);
char * iomeWhichItem(iomeConfig *CONF,char *key);

/* Default Configuration Loader */
typedef struct {
  iomeConfig SYS;
  iomeConfig CONF;
  iomeConfig MODULES;
} iomeDefaultConf;

int iomeDefaultLoader(iomeDefaultConf * DEFCONF);

/********************************************************\
                     Module functions
 --------------------------------------------------------
 Remember, modules are not allow to keep global variables
\********************************************************/
typedef char * iomeString;
typedef struct {
  char  * encoding;      /* big5, gb2312, ... */
  char  * class;         /* ime, filter */
  char ** accept_system; /* x,tty,null */
} iomeRegistry;

typedef struct {
  char * MethodName; /* This name should be load from CONF */
  char * FilterName; /* This name should be load from CONF */
  char * Encoding;   /* big5, gb2312, ... */
  char * ListArea;   /* Message for the ListArea, IME has to allocate itself, free it itself */
  char * KeyArea;    /* Message for the KeyArea */
  char * Value;      /* Return Value */
  int    CWidth;     /* Font Size */
  int    CHeight;
  int    EWidth;
  int    EHeight;
  int    lwidth,lheight; /* ListArea Range */
  int    kwidth,kheight; /* KeyArea Range */  
  void * Owner;      /* Owner's data */
} iomeRecord;

typedef int (*iomeEncoding)(unsigned char ch[]);
typedef int (*iomeRegister)(char *system,char *version,iomeRegistry *REG);
typedef int (*iomeInitialize)(iomeConfig *CONF,iomeRecord *RECORD);
typedef int (*iomePrepare)(iomeRecord *RECORD);
typedef int (*iomeFeedKey)(iomeRecord *RECORD,int ch,int state);
typedef int (*iomeMouseOverListArea)(iomeRecord *RECORD,int x,int y);
typedef int (*iomeMouseOverKeyArea)(iomeRecord *RECORD,int x,int y);
typedef int (*iomeNeedFocus)(iomeRecord *RECORD);
typedef int (*iomeSimpleConvert)(iomeRecord *RECORD,char *charset,char *value);
typedef int (*iomeQuit)(iomeRecord *RECORD);

#define IME_REGISTER "imeRegister"
#define IME_INIT     "imeInit"
#define IME_PREPARE  "imePrepare"
#define IME_FEEDKEY  "imeFeedKey"
#define IME_LISTAREA "imeListArea"
#define IME_KEYAREA  "imeKeyArea"
#define IME_FOCUS    "imeFocus"
#define IME_SIMPLE   "imeSimple"
#define IME_QUIT     "imeQuit"

typedef struct {
  iomeRecord            RECORD;
  iomeInitialize        INIT;
  iomePrepare           PREPARE;
  iomeFeedKey           FEEDKEY;
  iomeMouseOverListArea LISTAREA;
  iomeMouseOverKeyArea  KEYAREA;
  iomeNeedFocus         FOCUS;
  iomeSimpleConvert     SIMPLE;
  iomeQuit              QUIT;
} iomeModule;
/***********************************************\
  Before feedkey, convert to IME KEYCODE
\***********************************************/
#define IME_KEY_BACK   9
#define IME_KEY_DELETE 26
#define IME_KEY_ESC    27
#define IME_KEY_ENTER  13

#define IME_RT_REJECT   -1
#define IME_RT_BLOCKOUT  0
#define IME_RT_SEND      1
/***********************************************\
               Default functions
\***********************************************/
int isBig5(unsigned char ch[]);
int isGB2312(unsigned char ch[]);


#ifdef __cplusplus
}
#endif

#endif
