#include "keymap.h"
#include <linux/kd.h>
#include <linux/keyboard.h>

/**************************\
          Key Map
\**************************/

/* STATE */
#define KST_CTRL        (1<<KG_CTRL)
#define KST_ALT         (1<<KG_ALT)
#define KST_SHIFT       (1<<KG_SHIFT)
#define KST_CTRL_ALT    (KST_CTRL|KST_ALT)
#define KST_ALT_SHIFT   (KST_ALT|KST_SHIFT)

/* MAP */
#define KMP_DEL            0x7F

#define KMP_ALT_SHIFT_A    0x90
#define KMP_ALT_SHIFT_B    0x91
#define KMP_ALT_SHIFT_C    0x92
#define KMP_ALT_SHIFT_D    0x93
#define KMP_ALT_SHIFT_E    0x94
#define KMP_ALT_SHIFT_F    0x95
#define KMP_ALT_SHIFT_G    0x96
#define KMP_ALT_SHIFT_H    0x97
#define KMP_ALT_SHIFT_I    0x98
#define KMP_ALT_SHIFT_J    0x99
#define KMP_ALT_SHIFT_K    0x9A
#define KMP_ALT_SHIFT_L    0x9B
#define KMP_ALT_SHIFT_M    0x9C
#define KMP_ALT_SHIFT_N    0x9D
#define KMP_ALT_SHIFT_O    0x9E
#define KMP_ALT_SHIFT_P    0x9F
#define KMP_ALT_SHIFT_Q    0xA0
#define KMP_ALT_SHIFT_R    0xA1
#define KMP_ALT_SHIFT_S    0xA2
#define KMP_ALT_SHIFT_T    0xA3
#define KMP_ALT_SHIFT_U    0xA4
#define KMP_ALT_SHIFT_V    0xA5
#define KMP_ALT_SHIFT_W    0xA6
#define KMP_ALT_SHIFT_X    0xA7
#define KMP_ALT_SHIFT_Y    0xA8
#define KMP_ALT_SHIFT_Z    0xA9
#define KMP_ALT_SHIFT_1    0xAA
#define KMP_ALT_SHIFT_2    0xAB
#define KMP_ALT_SHIFT_3    0xAC
#define KMP_ALT_SHIFT_4    0xAD
#define KMP_ALT_SHIFT_5    0xAE
#define KMP_ALT_SHIFT_6    0xAF
#define KMP_ALT_SHIFT_7    0xB0
#define KMP_ALT_SHIFT_8    0xB1
#define KMP_ALT_SHIFT_9    0xB2
#define KMP_ALT_SHIFT_0    0xB3

#define KMP_CTRL_ALT_A     0xC0
#define KMP_CTRL_ALT_B     0xC1
#define KMP_CTRL_ALT_C     0xC2
#define KMP_CTRL_ALT_D     0xC3
#define KMP_CTRL_ALT_E     0xC4
#define KMP_CTRL_ALT_F     0xC5
#define KMP_CTRL_ALT_G     0xC6
#define KMP_CTRL_ALT_H     0xC7
#define KMP_CTRL_ALT_I     0xC8
#define KMP_CTRL_ALT_J     0xC9
#define KMP_CTRL_ALT_K     0xCA
#define KMP_CTRL_ALT_L     0xCB
#define KMP_CTRL_ALT_M     0xCC
#define KMP_CTRL_ALT_N     0xCD
#define KMP_CTRL_ALT_O     0xCE
#define KMP_CTRL_ALT_P     0xCF
#define KMP_CTRL_ALT_Q     0xD0
#define KMP_CTRL_ALT_R     0xD1
#define KMP_CTRL_ALT_S     0xD2
#define KMP_CTRL_ALT_T     0xD3
#define KMP_CTRL_ALT_U     0xD4
#define KMP_CTRL_ALT_V     0xD5
#define KMP_CTRL_ALT_W     0xD6
#define KMP_CTRL_ALT_X     0xD7
#define KMP_CTRL_ALT_Y     0xD8
#define KMP_CTRL_ALT_Z     0xD9
#define KMP_CTRL_ALT_1     0xDA
#define KMP_CTRL_ALT_2     0xDB
#define KMP_CTRL_ALT_3     0xDC
#define KMP_CTRL_ALT_4     0xDD
#define KMP_CTRL_ALT_5     0xDE
#define KMP_CTRL_ALT_6     0xDF
#define KMP_CTRL_ALT_7     0xE0
#define KMP_CTRL_ALT_8     0xE1
#define KMP_CTRL_ALT_9     0xE2
#define KMP_CTRL_ALT_0     0xE3

#define KMP_CTRL_ALT_BACK  0xED
#define KMP_CTRL_ALT_MARK  0xEE

#define KMP_INSERT         0xEF
#define KMP_HOME           0xF0
#define KMP_PGUP           0xF1
#define KMP_DELETE         0xF2
#define KMP_END            0xF3
#define KMP_PGDN           0xF4
#define KMP_UP             0xF5
#define KMP_DOWN           0xF6
#define KMP_LEFT           0xF7
#define KMP_RIGHT          0xF8

#define KMP_CTRL_SPACE     0xF9
#define KMP_CTRL_SHIFT     0xFA
#define KMP_CTRL_ESC       0xFB
#define KMP_CTRL_MARK      0xFC
#define KMP_CTRL_TAB       0xFD
#define KMP_SHIFT_SPACE    0xFE

/**************************\
        Scan Code
\**************************/
#define SCAN_ESC      0x01
#define SCAN_1        0x02
#define SCAN_2        0x03
#define SCAN_3        0x04
#define SCAN_4        0x05
#define SCAN_5        0x06
#define SCAN_6        0x07
#define SCAN_7        0x08
#define SCAN_8        0x09
#define SCAN_9        0x0A
#define SCAN_0        0x0B
#define SCAN_MINUS    0x0C
#define SCAN_PLUS     0x0D
#define SCAN_BACK     0x0E
#define SCAN_TAB      0x0F
#define SCAN_Q        0x10
#define SCAN_W        0x11
#define SCAN_E        0x12
#define SCAN_R        0x13
#define SCAN_T        0x14
#define SCAN_Y        0x15
#define SCAN_U        0x16
#define SCAN_I        0x17
#define SCAN_O        0x18
#define SCAN_P        0x19
#define SCAN_LTQUOTE  0x1A
#define SCAN_RTQUOTE  0x1B
#define SCAN_ENTER    0x1C
#define SCAN_CTRL     0x1D
#define SCAN_A        0x1E
#define SCAN_S        0x1F
#define SCAN_D        0x20
#define SCAN_F        0x21
#define SCAN_G        0x22
#define SCAN_H        0x23
#define SCAN_J        0x24
#define SCAN_K        0x25
#define SCAN_L        0x26
#define SCAN_SPLIT    0x27
#define SCAN_QUOTE    0x28
#define SCAN_MARK     0x29
#define SCAN_LSHIFT   0x2A
#define SCAN_STAND    0x2B
#define SCAN_Z        0x2C
#define SCAN_X        0x2D
#define SCAN_C        0x2E
#define SCAN_V        0x2F
#define SCAN_B        0x30
#define SCAN_N        0x31
#define SCAN_M        0x32
#define SCAN_LSQUOTE  0x33
#define SCAN_RSQUOTE  0x34
#define SCAN_QUESTION 0x35
#define SCAN_RSHIFT   0x36
#define SCAN_PRTSCR   0x37
#define SCAN_ALT      0x38
#define SCAN_SPACE    0x39
#define SCAN_CAPSLOCK 0x3A
#define SCAN_F1       0x3B
#define SCAN_F2       0x3C
#define SCAN_F3       0x3D
#define SCAN_F4       0x3E
#define SCAN_F5       0x3F
#define SCAN_F6       0x40
#define SCAN_F7       0x41
#define SCAN_F8       0x42
#define SCAN_F9       0x43
#define SCAN_F10      0x44
#define SCAN_NUMLOCK  0x45

#define SCAN_HOME     0x47
#define SCAN_UP       0x48
#define SCAN_PGUP     0x49
#define SCAN_LEFT     0x4B

#define SCAN_RIGHT    0x4D

#define SCAN_END      0x4F
#define SCAN_DOWN     0x50
#define SCAN_PGDN     0x51
#define SCAN_INSERT   0x52
#define SCAN_DELETE   0x53
#define SCAN_F11      0x85
#define SCAN_F12      0x86

#define KEYMAP_SIZE 94
struct kbentry SetKEYMAP[KEYMAP_SIZE]={
  {0,SCAN_ENTER,13},                       /* ENTER */
  {0,SCAN_BACK,KMP_DEL},                   /* BACK */
  {0,SCAN_TAB,9},                          /* TAB */
  {0,SCAN_ESC,27},                         /* ESC */

  {KST_CTRL,SCAN_SPACE,KMP_CTRL_SPACE},    /* CTRL+SPACE */
  {KST_CTRL,SCAN_LSHIFT,KMP_CTRL_SHIFT},   /* CTRL+SHIFT */
  {KST_CTRL,SCAN_ESC,KMP_CTRL_ESC},        /* CTRL+ESC */
  {KST_CTRL,SCAN_MARK,KMP_CTRL_MARK},      /* CTRL+MARK */
  {KST_CTRL,SCAN_TAB,KMP_CTRL_TAB},        /* CTRL+TAB */
  {KST_SHIFT,SCAN_SPACE,KMP_SHIFT_SPACE},  /* SHIFT+SPACE */

  {0,SCAN_INSERT,KMP_INSERT},
  {0,SCAN_HOME,KMP_HOME},
  {0,SCAN_PGUP,KMP_PGUP},
  {0,SCAN_DELETE,KMP_DELETE},
  {0,SCAN_END,KMP_END},
  {0,SCAN_PGDN,KMP_PGDN},
  {0,SCAN_UP,KMP_UP},
  {0,SCAN_DOWN,KMP_DOWN},
  {0,SCAN_LEFT,KMP_LEFT},
  {0,SCAN_RIGHT,KMP_RIGHT},

  {KST_CTRL_ALT,SCAN_MARK,KMP_CTRL_ALT_MARK},
  {KST_CTRL_ALT,SCAN_BACK,KMP_CTRL_ALT_BACK},

  {KST_CTRL_ALT,SCAN_0,KMP_CTRL_ALT_0},
  {KST_CTRL_ALT,SCAN_1,KMP_CTRL_ALT_1},
  {KST_CTRL_ALT,SCAN_2,KMP_CTRL_ALT_2},
  {KST_CTRL_ALT,SCAN_3,KMP_CTRL_ALT_3},
  {KST_CTRL_ALT,SCAN_4,KMP_CTRL_ALT_4},
  {KST_CTRL_ALT,SCAN_5,KMP_CTRL_ALT_5},
  {KST_CTRL_ALT,SCAN_6,KMP_CTRL_ALT_6},
  {KST_CTRL_ALT,SCAN_7,KMP_CTRL_ALT_7},
  {KST_CTRL_ALT,SCAN_8,KMP_CTRL_ALT_8},
  {KST_CTRL_ALT,SCAN_9,KMP_CTRL_ALT_9},
  {KST_CTRL_ALT,SCAN_A,KMP_CTRL_ALT_A},
  {KST_CTRL_ALT,SCAN_B,KMP_CTRL_ALT_B},
  {KST_CTRL_ALT,SCAN_C,KMP_CTRL_ALT_C},
  {KST_CTRL_ALT,SCAN_D,KMP_CTRL_ALT_D},
  {KST_CTRL_ALT,SCAN_E,KMP_CTRL_ALT_E},
  {KST_CTRL_ALT,SCAN_F,KMP_CTRL_ALT_F},
  {KST_CTRL_ALT,SCAN_G,KMP_CTRL_ALT_G},
  {KST_CTRL_ALT,SCAN_H,KMP_CTRL_ALT_H},
  {KST_CTRL_ALT,SCAN_I,KMP_CTRL_ALT_I},
  {KST_CTRL_ALT,SCAN_J,KMP_CTRL_ALT_J},
  {KST_CTRL_ALT,SCAN_K,KMP_CTRL_ALT_K},
  {KST_CTRL_ALT,SCAN_L,KMP_CTRL_ALT_L},
  {KST_CTRL_ALT,SCAN_M,KMP_CTRL_ALT_M},
  {KST_CTRL_ALT,SCAN_N,KMP_CTRL_ALT_N},
  {KST_CTRL_ALT,SCAN_O,KMP_CTRL_ALT_O},
  {KST_CTRL_ALT,SCAN_P,KMP_CTRL_ALT_P},
  {KST_CTRL_ALT,SCAN_Q,KMP_CTRL_ALT_Q},
  {KST_CTRL_ALT,SCAN_R,KMP_CTRL_ALT_R},
  {KST_CTRL_ALT,SCAN_S,KMP_CTRL_ALT_S},
  {KST_CTRL_ALT,SCAN_T,KMP_CTRL_ALT_T},
  {KST_CTRL_ALT,SCAN_U,KMP_CTRL_ALT_U},
  {KST_CTRL_ALT,SCAN_V,KMP_CTRL_ALT_V},
  {KST_CTRL_ALT,SCAN_W,KMP_CTRL_ALT_W},
  {KST_CTRL_ALT,SCAN_X,KMP_CTRL_ALT_X},
  {KST_CTRL_ALT,SCAN_Y,KMP_CTRL_ALT_Y},
  {KST_CTRL_ALT,SCAN_Z,KMP_CTRL_ALT_Z},

  {KST_ALT_SHIFT,SCAN_0,KMP_ALT_SHIFT_0},
  {KST_ALT_SHIFT,SCAN_1,KMP_ALT_SHIFT_1},
  {KST_ALT_SHIFT,SCAN_2,KMP_ALT_SHIFT_2},
  {KST_ALT_SHIFT,SCAN_3,KMP_ALT_SHIFT_3},
  {KST_ALT_SHIFT,SCAN_4,KMP_ALT_SHIFT_4},
  {KST_ALT_SHIFT,SCAN_5,KMP_ALT_SHIFT_5},
  {KST_ALT_SHIFT,SCAN_6,KMP_ALT_SHIFT_6},
  {KST_ALT_SHIFT,SCAN_7,KMP_ALT_SHIFT_7},
  {KST_ALT_SHIFT,SCAN_8,KMP_ALT_SHIFT_8},
  {KST_ALT_SHIFT,SCAN_9,KMP_ALT_SHIFT_9},
  {KST_ALT_SHIFT,SCAN_A,KMP_ALT_SHIFT_A},
  {KST_ALT_SHIFT,SCAN_B,KMP_ALT_SHIFT_B},
  {KST_ALT_SHIFT,SCAN_C,KMP_ALT_SHIFT_C},
  {KST_ALT_SHIFT,SCAN_D,KMP_ALT_SHIFT_D},
  {KST_ALT_SHIFT,SCAN_E,KMP_ALT_SHIFT_E},
  {KST_ALT_SHIFT,SCAN_F,KMP_ALT_SHIFT_F},
  {KST_ALT_SHIFT,SCAN_G,KMP_ALT_SHIFT_G},
  {KST_ALT_SHIFT,SCAN_H,KMP_ALT_SHIFT_H},
  {KST_ALT_SHIFT,SCAN_I,KMP_ALT_SHIFT_I},
  {KST_ALT_SHIFT,SCAN_J,KMP_ALT_SHIFT_J},
  {KST_ALT_SHIFT,SCAN_K,KMP_ALT_SHIFT_K},
  {KST_ALT_SHIFT,SCAN_L,KMP_ALT_SHIFT_L},
  {KST_ALT_SHIFT,SCAN_M,KMP_ALT_SHIFT_M},
  {KST_ALT_SHIFT,SCAN_N,KMP_ALT_SHIFT_N},
  {KST_ALT_SHIFT,SCAN_O,KMP_ALT_SHIFT_O},
  {KST_ALT_SHIFT,SCAN_P,KMP_ALT_SHIFT_P},
  {KST_ALT_SHIFT,SCAN_Q,KMP_ALT_SHIFT_Q},
  {KST_ALT_SHIFT,SCAN_R,KMP_ALT_SHIFT_R},
  {KST_ALT_SHIFT,SCAN_S,KMP_ALT_SHIFT_S},
  {KST_ALT_SHIFT,SCAN_T,KMP_ALT_SHIFT_T},
  {KST_ALT_SHIFT,SCAN_U,KMP_ALT_SHIFT_U},
  {KST_ALT_SHIFT,SCAN_V,KMP_ALT_SHIFT_V},
  {KST_ALT_SHIFT,SCAN_W,KMP_ALT_SHIFT_W},
  {KST_ALT_SHIFT,SCAN_X,KMP_ALT_SHIFT_X},
  {KST_ALT_SHIFT,SCAN_Y,KMP_ALT_SHIFT_Y},
  {KST_ALT_SHIFT,SCAN_Z,KMP_ALT_SHIFT_Z},

};

void ttySetupKeymap(int console)
{
  int n;
  for (n=0;n<KEYMAP_SIZE;n++) 
    ioctl(console,KDSKBENT,&SetKEYMAP[n]);
}

typedef struct {
  int key;
  int iome;
  int state;
} ttyKEYMAP;

#define KFT_CTRL      IME_KF_CONTROL
#define KFT_SHIFT     IME_KF_SHIFT
#define KFT_ALT       IME_KF_ALT
#define KFT_CTRL_ALT  (IME_KF_ALT|IME_KF_CONTROL)
#define KFT_ALT_SHIFT (IME_KF_ALT|IME_KF_SHIFT)

#define TTY2IOME_MAP 95
ttyKEYMAP tty2iome[TTY2IOME_MAP]={
  {KMP_DEL            ,IME_KEY_BACK    ,0},
  {KMP_CTRL_SPACE     ,' '             ,KFT_CTRL},
  {KMP_CTRL_SHIFT     ,0               ,(KFT_CTRL|KFT_SHIFT)},
  {KMP_CTRL_ESC       ,IME_KEY_ESC     ,KFT_CTRL},
  {KMP_CTRL_MARK      ,'~'             ,KFT_CTRL},
  {KMP_CTRL_TAB       ,IME_KEY_TAB     ,KFT_CTRL},
  {KMP_SHIFT_SPACE    ,' '             ,KFT_SHIFT},

  {KMP_CTRL_ALT_A     ,'a'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_B     ,'b'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_C     ,'c'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_D     ,'d'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_E     ,'e'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_F     ,'f'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_G     ,'g'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_H     ,'h'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_I     ,'i'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_J     ,'j'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_K     ,'k'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_L     ,'l'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_M     ,'n'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_N     ,'m'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_O     ,'o'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_P     ,'p'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_Q     ,'q'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_R     ,'r'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_S     ,'s'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_T     ,'t'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_U     ,'u'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_V     ,'v'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_W     ,'w'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_X     ,'x'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_Y     ,'y'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_Z     ,'z'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_1     ,'1'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_2     ,'2'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_3     ,'3'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_4     ,'4'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_5     ,'5'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_6     ,'6'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_7     ,'7'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_8     ,'8'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_9     ,'9'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_0     ,'0'             ,KFT_CTRL_ALT},

  {KMP_ALT_SHIFT_A    ,'a'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_B    ,'b'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_C    ,'c'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_D    ,'d'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_E    ,'e'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_F    ,'f'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_G    ,'g'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_H    ,'h'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_I    ,'i'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_J    ,'j'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_K    ,'k'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_L    ,'l'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_M    ,'n'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_N    ,'m'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_O    ,'o'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_P    ,'p'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_Q    ,'q'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_R    ,'r'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_S    ,'s'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_T    ,'t'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_U    ,'u'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_V    ,'v'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_W    ,'w'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_X    ,'x'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_Y    ,'y'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_Z    ,'z'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_1    ,'1'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_2    ,'2'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_3    ,'3'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_4    ,'4'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_5    ,'5'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_6    ,'6'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_7    ,'7'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_8    ,'8'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_9    ,'9'             ,KFT_ALT_SHIFT},
  {KMP_ALT_SHIFT_0    ,'0'             ,KFT_ALT_SHIFT},

  {KMP_INSERT         ,IME_KEY_INSERT  ,0},
  {KMP_HOME           ,IME_KEY_HOME    ,0},
  {KMP_PGUP           ,IME_KEY_PAGEUP  ,0},
  {KMP_DELETE         ,IME_KEY_DELETE  ,0},
  {KMP_END            ,IME_KEY_END     ,0},
  {KMP_PGDN           ,IME_KEY_PAGEDOWN,0},
  {KMP_UP             ,IME_KEY_UP      ,0},
  {KMP_DOWN           ,IME_KEY_DOWN    ,0},
  {KMP_LEFT           ,IME_KEY_LEFT    ,0},
  {KMP_RIGHT          ,IME_KEY_RIGHT   ,0},
  {KMP_CTRL_ALT_MARK  ,'~'             ,KFT_CTRL_ALT},
  {KMP_CTRL_ALT_BACK  ,IME_KEY_BACK    ,KFT_CTRL_ALT},

  {27                 ,IME_KEY_ESC     ,0},
  {9                  ,IME_KEY_TAB     ,0},
  {13                 ,IME_KEY_ENTER   ,0},
  {8                  ,IME_KEY_BACK    ,0},

};

int ttyConvertKey(int ch,int *key,int *state)
{
  int n;
  for (n=0;n<TTY2IOME_MAP;n++)
    if (ch==tty2iome[n].key) {
      *key = tty2iome[n].iome;
      *state = tty2iome[n].state;
      return ch;
    }
  *key = ch;
  *state = 0;
  return ch;
}

